@@ -2,30 +2,33 @@
 from __future__ import unicode_literals
 
 MESSAGES = {
-    "Also available in": "Saatavilla myös",
+    "%d min remaining to read": "",
     "Also available in:": "Saatavilla myös:",
     "Archive": "Arkisto",
     "Categories": "Kategoriat",
+    "Comments": "Kommentit",
     "LANGUAGE": "Suomi",
+    "Languages:": "Kielet:",
     "More posts about %s": "Lisää postauksia aiheesta %s",
-    "More posts about": "Lisää postauksia aiheesta",
     "Newer posts": "Uudempia postauksia",
     "Next post": "Seuraava postaus",
-    "No posts found.": "",
-    "Nothing found.": "",
+    "No posts found.": "Postauksia ei löytynyt.",
+    "Nothing found.": "Ei hakutuloksia.",
     "Older posts": "Vanhempia postauksia",
     "Original site": "Alkuperäinen sivusto",
-    "Posted": "Postattu",
     "Posted:": "Postattu:",
     "Posts about %s": "Postauksia aiheesta %s",
     "Posts for year %s": "Postauksia vuodelta %s",
     "Posts for {month} {year}": "Postauksia ajalle {month} {year}",
     "Previous post": "Vanhempia postauksia",
+    "Publication date": "Julkaisupäivämäärä",
+    "RSS feed": "RSS syöte",
     "Read in English": "Lue suomeksi",
     "Read more": "Lue lisää",
+    "Skip to main content": "",
     "Source": "Lähde",
     "Tags and Categories": "Tagit ja kategoriat",
     "Tags": "Tagit",
-    "old posts, page %d": "vanhojen postauksien, sivu %d",
+    "old posts, page %d": "vanhoja postauksia, sivu %d",
     "page %d": "sivu %d",
 }
