@@ -25,7 +25,7 @@
 # SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 from __future__ import print_function, unicode_literals
-import codecs
+import io
 from collections import defaultdict
 from copy import copy
 from pkg_resources import resource_filename
@@ -33,7 +33,9 @@ import datetime
 import glob
 import locale
 import os
+import json
 import sys
+import mimetypes
 try:
     from urlparse import urlparse, urlsplit, urljoin
 except ImportError:
@@ -78,6 +80,7 @@ from .plugin_categories import (
     TaskMultiplier,
     TemplateSystem,
     SignalHandler,
+    ConfigPlugin,
 )
 
 
@@ -133,7 +136,7 @@ LEGAL_VALUES = {
         'pt': 'pt_br',
         'zh': 'zh_cn'
     },
-    'RTL_LANGUAGES': ('fa', 'ur'),
+    'RTL_LANGUAGES': ('ar', 'fa', 'ur'),
     'COLORBOX_LOCALES': defaultdict(
         str,
         bg='bg',
@@ -159,10 +162,46 @@ LEGAL_VALUES = {
         sl='si',  # country code is si, language code is sl, colorbox is wrong
         tr='tr',
         zh_cn='zh-CN'
-    )
+    ),
+    'MOMENTJS_LOCALES': defaultdict(
+        str,
+        bg='bg',
+        ca='ca',
+        cs='cs',
+        cz='cs',
+        de='de',
+        en='',
+        es='es',
+        et='et',
+        fa='fa',
+        fi='fi',
+        fr='fr',
+        hr='hr',
+        it='it',
+        ja='ja',
+        nb='nb',
+        nl='nl',
+        pt_br='pt-br',
+        pl='pl',
+        ru='ru',
+        sk='sk',
+        sl='sl',
+        tr='tr',
+        zh_cn='zh-cn'
+    ),
 }
 
 
+def _enclosure(post, lang):
+    '''Default implementation of enclosures'''
+    enclosure = post.meta('enclosure', lang)
+    if enclosure:
+        length = 0
+        url = enclosure
+        mime = mimetypes.guess_type(url)[0]
+        return url, length, mime
+
+
 class Nikola(object):
 
     """Class that handles site generation.
@@ -242,8 +281,10 @@ class Nikola(object):
             'CREATE_MONTHLY_ARCHIVE': False,
             'CREATE_SINGLE_ARCHIVE': False,
             'DATE_FORMAT': '%Y-%m-%d %H:%M',
+            'JS_DATE_FORMAT': 'YYYY-MM-DD HH:mm',
+            'DATE_FANCINESS': 0,
             'DEFAULT_LANG': "en",
-            'DEPLOY_COMMANDS': [],
+            'DEPLOY_COMMANDS': {'default': []},
             'DISABLED_PLUGINS': [],
             'EXTRA_PLUGINS_DIRS': [],
             'COMMENT_SYSTEM_ID': 'nikolademo',
@@ -257,6 +298,7 @@ class Nikola(object):
             'FORCE_ISO8601': False,
             'GALLERY_PATH': 'galleries',
             'GALLERY_SORT_BY_DATE': True,
+            'GLOBAL_CONTEXT_FILLER': [],
             'GZIP_COMMAND': None,
             'GZIP_FILES': False,
             'GZIP_EXTENSIONS': ('.txt', '.htm', '.html', '.css', '.js', '.json', '.xml'),
@@ -317,8 +359,10 @@ class Nikola(object):
             'URL_TYPE': 'rel_path',
             'USE_BUNDLES': True,
             'USE_CDN': False,
+            'USE_CDN_WARNING': True,
             'USE_FILENAME_AS_TITLE': True,
             'USE_OPEN_GRAPH': True,
+            'USE_SLUGIFY': True,
             'TIMEZONE': 'UTC',
             'DEPLOY_DRAFTS': True,
             'DEPLOY_FUTURE': False,
@@ -389,6 +433,9 @@ class Nikola(object):
         # We provide the arguments to format in CONTENT_FOOTER_FORMATS.
         self.config['CONTENT_FOOTER'].langformat(self.config['CONTENT_FOOTER_FORMATS'])
 
+        # propagate USE_SLUGIFY
+        utils.USE_SLUGIFY = self.config['USE_SLUGIFY']
+
         # Make sure we have pyphen installed if we are using it
         if self.config.get('HYPHENATE') and pyphen is None:
             utils.LOGGER.warn('To use the hyphenation, you have to install '
@@ -481,7 +528,15 @@ class Nikola(object):
             self.config['BASE_URL'] = self.config.get('SITE_URL')
         # BASE_URL should *always* end in /
         if self.config['BASE_URL'] and self.config['BASE_URL'][-1] != '/':
-            utils.LOGGER.warn("Your BASE_URL doesn't end in / -- adding it.")
+            utils.LOGGER.warn("Your BASE_URL doesn't end in / -- adding it, but please fix it in your config file!")
+
+        # todo: remove in v8
+        if not isinstance(self.config['DEPLOY_COMMANDS'], dict):
+            utils.LOGGER.warn("A single list as DEPLOY_COMMANDS is deprecated.  DEPLOY_COMMANDS should be a dict, with deploy preset names as keys and lists of commands as values.")
+            utils.LOGGER.warn("The key `default` is used by `nikola deploy`:")
+            self.config['DEPLOY_COMMANDS'] = {'default': self.config['DEPLOY_COMMANDS']}
+            utils.LOGGER.warn("DEPLOY_COMMANDS = {0}".format(self.config['DEPLOY_COMMANDS']))
+            utils.LOGGER.info("(The above can be used with `nikola deploy` or `nikola deploy default`.  Multiple presets are accepted.)")
 
         # We use one global tzinfo object all over Nikola.
         self.tzinfo = dateutil.tz.gettz(self.config['TIMEZONE'])
@@ -497,6 +552,7 @@ class Nikola(object):
             "RestExtension": RestExtension,
             "MarkdownExtension": MarkdownExtension,
             "SignalHandler": SignalHandler,
+            "ConfigPlugin": ConfigPlugin,
         })
         self.plugin_manager.setPluginInfoExtension('plugin')
         extra_plugins_dirs = self.config['EXTRA_PLUGINS_DIRS']
@@ -516,45 +572,21 @@ class Nikola(object):
         self.plugin_manager.setPluginPlaces(places)
         self.plugin_manager.collectPlugins()
 
-        # Activate all required SignalHandler plugins
-        for plugin_info in self.plugin_manager.getPluginsOfCategory("SignalHandler"):
-            if plugin_info.name in self.config.get('DISABLED_PLUGINS'):
-                self.plugin_manager.removePluginFromCategory(plugin_info, "SignalHandler")
-            else:
-                self.plugin_manager.activatePluginByName(plugin_info.name)
-                plugin_info.plugin_object.set_site(self)
+        self._activate_plugins_of_category("SignalHandler")
 
         # Emit signal for SignalHandlers which need to start running immediately.
         signal('sighandlers_loaded').send(self)
 
         self._commands = {}
-        # Activate all command plugins
-        for plugin_info in self.plugin_manager.getPluginsOfCategory("Command"):
-            if plugin_info.name in self.config['DISABLED_PLUGINS']:
-                self.plugin_manager.removePluginFromCategory(plugin_info, "Command")
-                continue
 
-            self.plugin_manager.activatePluginByName(plugin_info.name)
-            plugin_info.plugin_object.set_site(self)
+        command_plugins = self._activate_plugins_of_category("Command")
+        for plugin_info in command_plugins:
             plugin_info.plugin_object.short_help = plugin_info.description
             self._commands[plugin_info.name] = plugin_info.plugin_object
 
-        # Activate all task plugins
-        for task_type in ["Task", "LateTask"]:
-            for plugin_info in self.plugin_manager.getPluginsOfCategory(task_type):
-                if plugin_info.name in self.config['DISABLED_PLUGINS']:
-                    self.plugin_manager.removePluginFromCategory(plugin_info, task_type)
-                    continue
-                self.plugin_manager.activatePluginByName(plugin_info.name)
-                plugin_info.plugin_object.set_site(self)
-
-        # Activate all multiplier plugins
-        for plugin_info in self.plugin_manager.getPluginsOfCategory("TaskMultiplier"):
-            if plugin_info.name in self.config['DISABLED_PLUGINS']:
-                self.plugin_manager.removePluginFromCategory(plugin_info, task_type)
-                continue
-            self.plugin_manager.activatePluginByName(plugin_info.name)
-            plugin_info.plugin_object.set_site(self)
+        self._activate_plugins_of_category("Task")
+        self._activate_plugins_of_category("LateTask")
+        self._activate_plugins_of_category("TaskMultiplier")
 
         compilers = defaultdict(set)
         # Also add aliases for combinations with TRANSLATIONS_PATTERN
@@ -592,8 +624,7 @@ class Nikola(object):
         self._GLOBAL_CONTEXT['use_bundles'] = self.config['USE_BUNDLES']
         self._GLOBAL_CONTEXT['use_cdn'] = self.config.get("USE_CDN")
         self._GLOBAL_CONTEXT['favicons'] = self.config['FAVICONS']
-        self._GLOBAL_CONTEXT['date_format'] = self.config.get(
-            'DATE_FORMAT', '%Y-%m-%d %H:%M')
+        self._GLOBAL_CONTEXT['date_format'] = self.config.get('DATE_FORMAT')
         self._GLOBAL_CONTEXT['blog_author'] = self.config.get('BLOG_AUTHOR')
         self._GLOBAL_CONTEXT['blog_title'] = self.config.get('BLOG_TITLE')
         self._GLOBAL_CONTEXT['show_blog_title'] = self.config.get('SHOW_BLOG_TITLE')
@@ -634,7 +665,11 @@ class Nikola(object):
         self._GLOBAL_CONTEXT['show_sourcelink'] = self.config.get(
             'SHOW_SOURCELINK')
         self._GLOBAL_CONTEXT['extra_head_data'] = self.config.get('EXTRA_HEAD_DATA')
+        self._GLOBAL_CONTEXT['date_fanciness'] = self.config.get('DATE_FANCINESS')
+        self._GLOBAL_CONTEXT['js_date_format'] = json.dumps(self.config.get('JS_DATE_FORMAT'))
         self._GLOBAL_CONTEXT['colorbox_locales'] = LEGAL_VALUES['COLORBOX_LOCALES']
+        self._GLOBAL_CONTEXT['momentjs_locales'] = LEGAL_VALUES['MOMENTJS_LOCALES']
+        self._GLOBAL_CONTEXT['url_replacer'] = self.url_replacer
 
         self._GLOBAL_CONTEXT.update(self.config.get('GLOBAL_CONTEXT', {}))
 
@@ -647,8 +682,22 @@ class Nikola(object):
             self.compilers[plugin_info.name] = \
                 plugin_info.plugin_object
 
+        self._activate_plugins_of_category("ConfigPlugin")
+
         signal('configured').send(self)
 
+    def _activate_plugins_of_category(self, category):
+        """Activate all the plugins of a given category and return them."""
+        plugins = []
+        for plugin_info in self.plugin_manager.getPluginsOfCategory(category):
+            if plugin_info.name in self.config.get('DISABLED_PLUGINS'):
+                self.plugin_manager.removePluginFromCategory(plugin_info, category)
+            else:
+                self.plugin_manager.activatePluginByName(plugin_info.name)
+                plugin_info.plugin_object.set_site(self)
+                plugins.append(plugin_info)
+        return plugins
+
     def _get_themes(self):
         if self._THEMES is None:
             try:
@@ -658,7 +707,7 @@ class Nikola(object):
                 self.config['THEME'] = 'bootstrap'
                 return self._get_themes()
             # Check consistency of USE_CDN and the current THEME (Issue #386)
-            if self.config['USE_CDN']:
+            if self.config['USE_CDN'] and self.config['USE_CDN_WARNING']:
                 bootstrap_path = utils.get_asset_path(os.path.join(
                     'assets', 'css', 'bootstrap.min.css'), self._THEMES)
                 if bootstrap_path and bootstrap_path.split(os.sep)[-4] not in ['bootstrap', 'bootstrap3']:
@@ -765,6 +814,9 @@ class Nikola(object):
         for h in local_context['template_hooks'].values():
             h.context = context
 
+        for func in self.config['GLOBAL_CONTEXT_FILLER']:
+            func(local_context, template_name)
+
         data = self.template_system.render_template(
             template_name, None, local_context)
 
@@ -866,6 +918,10 @@ class Nikola(object):
         if not result:
             result = "."
 
+        # Don't forget the query part of the link
+        if parsed_dst.query:
+            result += "?" + parsed_dst.query
+
         # Don't forget the fragment (anchor) part of the link
         if parsed_dst.fragment:
             result += "#" + parsed_dst.fragment
@@ -875,10 +931,10 @@ class Nikola(object):
         return result
 
     def generic_rss_renderer(self, lang, title, link, description, timeline, output_path,
-                             rss_teasers, rss_plain, feed_length=10, feed_url=None, enclosure=None):
+                             rss_teasers, rss_plain, feed_length=10, feed_url=None, enclosure=_enclosure):
 
         """Takes all necessary data, and renders a RSS feed in output_path."""
-        rss_obj = rss.RSS2(
+        rss_obj = utils.ExtendedRSS2(
             title=title,
             link=link,
             description=description,
@@ -887,6 +943,9 @@ class Nikola(object):
             language=lang
         )
 
+        if feed_url:
+            rss_obj.xsl_stylesheet_href = self.url_replacer(feed_url, "/assets/xml/rss.xsl")
+
         items = []
 
         for post in timeline[:feed_length]:
@@ -928,23 +987,21 @@ class Nikola(object):
             if post.author(lang):
                 rss_obj.rss_attrs["xmlns:dc"] = "http://purl.org/dc/elements/1.1/"
 
-            """ Enclosure callback must returns tuple """
-            if enclosure:
-                download_link, download_size, download_type = enclosure(post=post, lang=lang)
-
-                args['enclosure'] = rss.Enclosure(
-                    download_link,
-                    download_size,
-                    download_type,
-                )
+            # enclosure callback returns None if post has no enclosure, or a
+            # 3-tuple of (url, length (0 is valid), mimetype)
+            enclosure_details = enclosure(post=post, lang=lang)
+            if enclosure_details is not None:
+                args['enclosure'] = rss.Enclosure(*enclosure_details)
 
             items.append(utils.ExtendedItem(**args))
 
         rss_obj.items = items
+        rss_obj.self_url = feed_url
+        rss_obj.rss_attrs["xmlns:atom"] = "http://www.w3.org/2005/Atom"
 
         dst_dir = os.path.dirname(output_path)
         utils.makedirs(dst_dir)
-        with codecs.open(output_path, "wb+", "utf-8") as rss_file:
+        with io.open(output_path, "w+", encoding="utf-8") as rss_file:
             data = rss_obj.to_xml(encoding='utf-8')
             if isinstance(data, utils.bytes_str):
                 data = data.decode('utf-8')
@@ -1127,7 +1184,10 @@ class Nikola(object):
         if self._scanned and not really:
             return
 
-        self.commands = utils.Commands(self.doit)
+        try:
+            self.commands = utils.Commands(self.doit)
+        except AttributeError:
+            self.commands = None
         self.global_data = {}
         self.posts = []
         self.posts_per_year = defaultdict(list)
@@ -1199,16 +1259,17 @@ class Nikola(object):
                         self.posts_per_month[
                             '{0}/{1:02d}'.format(post.date.year, post.date.month)].append(post)
                         for tag in post.alltags:
-                            if utils.slugify(tag) in slugged_tags:
+                            _tag_slugified = utils.slugify(tag)
+                            if _tag_slugified in slugged_tags:
                                 if tag not in self.posts_per_tag:
                                     # Tags that differ only in case
-                                    other_tag = [k for k in self.posts_per_tag.keys() if k.lower() == tag.lower()][0]
+                                    other_tag = [existing for existing in self.posts_per_tag.keys() if utils.slugify(existing) == _tag_slugified][0]
                                     utils.LOGGER.error('You have tags that are too similar: {0} and {1}'.format(tag, other_tag))
                                     utils.LOGGER.error('Tag {0} is used in: {1}'.format(tag, post.source_path))
                                     utils.LOGGER.error('Tag {0} is used in: {1}'.format(other_tag, ', '.join([p.source_path for p in self.posts_per_tag[other_tag]])))
                                     quit = True
                             else:
-                                slugged_tags.add(utils.slugify(tag))
+                                slugged_tags.add(utils.slugify(tag, force=True))
                             self.posts_per_tag[tag].append(post)
                         self.posts_per_category[post.meta('category')].append(post)
                     else:
@@ -1249,7 +1310,6 @@ class Nikola(object):
         context['title'] = post.title(lang)
         context['description'] = post.description(lang)
         context['permalink'] = post.permalink(lang)
-        context['page_list'] = self.pages
         if post.use_in_feeds:
             context['enable_comments'] = True
         else:
@@ -1274,6 +1334,8 @@ class Nikola(object):
         for k in self._GLOBAL_CONTEXT_TRANSLATABLE:
             deps_dict[k] = deps_dict['global'][k](lang)
 
+        deps_dict['navigation_links'] = deps_dict['global']['navigation_links'](lang)
+
         if post:
             deps_dict['post_translations'] = post.translated_to
 
@@ -1315,6 +1377,8 @@ class Nikola(object):
         for k in self._GLOBAL_CONTEXT_TRANSLATABLE:
             deps_context[k] = deps_context['global'][k](lang)
 
+        deps_context['navigation_links'] = deps_context['global']['navigation_links'](lang)
+
         task = {
             'name': os.path.normpath(output_name),
             'targets': [output_name],
