@@ -28,7 +28,6 @@ from __future__ import print_function, unicode_literals
 
 import os
 import subprocess
-import webbrowser
 
 from nikola.plugin_categories import Command
 from nikola.utils import req_missing
@@ -61,7 +60,7 @@ class CommandAuto(Command):
         try:
             from livereload import Server
         except ImportError:
-            req_missing(['livereload==2.1.0'], 'use the "auto" command')
+            req_missing(['livereload'], 'use the "auto" command')
             return
 
         # Run an initial build so we are up-to-date
@@ -73,14 +72,16 @@ class CommandAuto(Command):
         server.watch('conf.py', 'nikola build')
         server.watch('themes/', 'nikola build')
         server.watch('templates/', 'nikola build')
-        server.watch(self.site.config['GALLERY_PATH'])
+        server.watch(self.site.config['GALLERY_PATH'], 'nikola build')
         for item in self.site.config['post_pages']:
             server.watch(os.path.dirname(item[0]), 'nikola build')
         for item in self.site.config['FILES_FOLDERS']:
-            server.watch(os.path.dirname(item), 'nikola build')
+            server.watch(item, 'nikola build')
 
         out_folder = self.site.config['OUTPUT_FOLDER']
         if options and options.get('browser'):
-            webbrowser.open('http://localhost:{0}'.format(port))
+            browser = True
+        else:
+            browser = False
 
-        server.serve(port, None, out_folder)
+        server.serve(port=port, host=None, root=out_folder, debug=True, open_url=browser)
