@@ -82,9 +82,9 @@ class CommandBootswatchTheme(Command):
 
         # See if we need bootswatch for bootstrap v2 or v3
         themes = utils.get_theme_chain(parent)
-        if 'bootstrap3' not in themes:
+        if 'bootstrap3' not in themes and 'bootstrap3-jinja' not in themes:
             version = '2'
-        elif 'bootstrap' not in themes:
+        elif 'bootstrap' not in themes and 'bootstrap-jinja' not in themes:
             LOGGER.warn('"bootswatch_theme" only makes sense for themes that use bootstrap')
         elif 'bootstrap3-gradients' in themes or 'bootstrap3-gradients-jinja' in themes:
             LOGGER.warn('"bootswatch_theme" doesn\'t work well with the bootstrap3-gradients family')
@@ -92,7 +92,10 @@ class CommandBootswatchTheme(Command):
         LOGGER.info("Creating '{0}' theme from '{1}' and '{2}'".format(name, swatch, parent))
         utils.makedirs(os.path.join('themes', name, 'assets', 'css'))
         for fname in ('bootstrap.min.css', 'bootstrap.css'):
-            url = '/'.join(('http://bootswatch.com', version, swatch, fname))
+            url = 'http://bootswatch.com'
+            if version:
+                url += '/' + version
+            url = '/'.join((url, swatch, fname))
             LOGGER.info("Downloading: " + url)
             data = requests.get(url).text
             with open(os.path.join('themes', name, 'assets', 'css', fname),
