@@ -30,9 +30,9 @@ import re
 import sys
 try:
     from urllib import unquote
-    from urlparse import urlparse
+    from urlparse import urlparse, urljoin, urldefrag
 except ImportError:
-    from urllib.parse import unquote, urlparse  # NOQA
+    from urllib.parse import unquote, urlparse, urljoin, urldefrag  # NOQA
 
 import lxml.html
 
@@ -51,7 +51,7 @@ def real_scan_files(site):
             fname = task.split(':', 1)[-1]
             task_fnames.add(fname)
     # And now check that there are no non-target files
-    for root, dirs, files in os.walk(output_folder):
+    for root, dirs, files in os.walk(output_folder, followlinks=True):
         for src_name in files:
             fname = os.path.join(root, src_name)
             real_fnames.add(fname)
@@ -63,6 +63,15 @@ def real_scan_files(site):
     return (only_on_output, only_on_input)
 
 
+def fs_relpath_from_url_path(url_path):
+    """Expects as input an urlparse(s).path"""
+    url_path = unquote(url_path)
+    # in windows relative paths don't begin with os.sep
+    if sys.platform == 'win32' and len(url_path):
+        url_path = url_path[1:].replace('/', '\\')
+    return url_path
+
+
 class CommandCheck(Command):
     """Check the generated site."""
 
@@ -139,16 +148,33 @@ class CommandCheck(Command):
         rv = False
         self.whitelist = [re.compile(x) for x in self.site.config['LINK_CHECK_WHITELIST']]
         base_url = urlparse(self.site.config['BASE_URL'])
+        self.existing_targets.add(self.site.config['SITE_URL'])
+        self.existing_targets.add(self.site.config['BASE_URL'])
         url_type = self.site.config['URL_TYPE']
+        if url_type == 'absolute':
+            url_netloc_to_root = urlparse(self.site.config['SITE_URL']).path
         try:
             filename = task.split(":")[-1]
+
+            if filename.startswith(self.site.config['CACHE_FOLDER']):
+                # Do not look at links in the cache, which are not parsed by
+                # anyone and may result in false positives.  Problems arise
+                # with galleries, for example.  Full rationale: (Issue #1447)
+                self.logger.notice("Ignoring {0} (in cache, links may be incorrect)".format(filename))
+                return False
+
             d = lxml.html.fromstring(open(filename).read())
             for l in d.iterlinks():
                 target = l[0].attrib[l[1]]
                 if target == "#":
                     continue
+                target, _ = urldefrag(target)
                 parsed = urlparse(target)
 
+                # Warn about links from https to http (mixed-security)
+                if base_url.netloc == parsed.netloc and base_url.scheme == "https" and parsed.scheme == "http":
+                    self.logger.warn("Mixed-content security for link in {0}: {1}".format(filename, target))
+
                 # Absolute links when using only paths, skip.
                 if (parsed.scheme or target.startswith('//')) and url_type in ('rel_path', 'full_path'):
                     continue
@@ -157,20 +183,20 @@ class CommandCheck(Command):
                 if (parsed.scheme or target.startswith('//')) and parsed.netloc != base_url.netloc:
                     continue
 
-                if parsed.fragment:
-                    target = target.split('#')[0]
                 if url_type == 'rel_path':
                     target_filename = os.path.abspath(
                         os.path.join(os.path.dirname(filename), unquote(target)))
 
                 elif url_type in ('full_path', 'absolute'):
-                    target_filename = os.path.abspath(
-                        os.path.join(os.path.dirname(filename), parsed.path))
-                    if parsed.path.endswith('/'):  # abspath removes trailing slashes
-                        target_filename += '/{0}'.format(self.site.config['INDEX_FILE'])
-                    if target_filename.startswith(base_url.path):
-                        target_filename = target_filename[len(base_url.path):]
-                    target_filename = os.path.join(self.site.config['OUTPUT_FOLDER'], target_filename)
+                    if url_type == 'absolute':
+                        # convert to 'full_path' case, ie url relative to root
+                        url_rel_path = target.path[len(url_netloc_to_root):]
+                    else:
+                        url_rel_path = target.path
+                    if url_rel_path == '' or url_rel_path.endswith('/'):
+                        url_rel_path = urljoin(url_rel_path, self.site.config['INDEX_FILE'])
+                    fs_rel_path = fs_relpath_from_url_path(url_rel_path)
+                    target_filename = os.path.join(self.site.config['OUTPUT_FOLDER'], fs_rel_path)
 
                 if any(re.match(x, target_filename) for x in self.whitelist):
                     continue
@@ -233,7 +259,7 @@ class CommandCheck(Command):
         return failure
 
     def clean_files(self):
-        only_on_output, _ = self.real_scan_files()
+        only_on_output, _ = real_scan_files(self.site)
         for f in only_on_output:
             os.unlink(f)
         return True
