@@ -25,8 +25,9 @@
 # SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 from __future__ import print_function
-import codecs
+import io
 from datetime import datetime
+from dateutil.tz import gettz
 import os
 import sys
 import subprocess
@@ -35,16 +36,16 @@ import time
 from blinker import signal
 
 from nikola.plugin_categories import Command
-from nikola.utils import remove_file, get_logger
+from nikola.utils import get_logger, remove_file, unicode_str
 
 
 class CommandDeploy(Command):
     """Deploy site."""
     name = "deploy"
 
-    doc_usage = ""
+    doc_usage = "[[preset [preset...]]"
     doc_purpose = "deploy the site"
-
+    doc_description = "Deploy the site by executing deploy commands from the presets listed on the command line.  If no presets are specified, `default` is executed."
     logger = None
 
     def _execute(self, command, args):
@@ -73,18 +74,33 @@ class CommandDeploy(Command):
                     remove_file(os.path.join(out_dir, post.source_path))
                     undeployed_posts.append(post)
 
-        for command in self.site.config['DEPLOY_COMMANDS']:
-            self.logger.info("==> {0}".format(command))
+        if args:
+            presets = args
+        else:
+            presets = ['default']
+
+        # test for preset existence
+        for preset in presets:
             try:
-                subprocess.check_call(command, shell=True)
-            except subprocess.CalledProcessError as e:
-                self.logger.error('Failed deployment — command {0} '
-                                  'returned {1}'.format(e.cmd, e.returncode))
-                sys.exit(e.returncode)
+                self.site.config['DEPLOY_COMMANDS'][preset]
+            except:
+                self.logger.error('No such preset: {0}'.format(preset))
+                sys.exit(255)
+
+        for preset in presets:
+            self.logger.info("=> preset '{0}'".format(preset))
+            for command in self.site.config['DEPLOY_COMMANDS'][preset]:
+                self.logger.info("==> {0}".format(command))
+                try:
+                    subprocess.check_call(command, shell=True)
+                except subprocess.CalledProcessError as e:
+                    self.logger.error('Failed deployment — command {0} '
+                                      'returned {1}'.format(e.cmd, e.returncode))
+                    sys.exit(e.returncode)
 
         self.logger.info("Successful deployment")
         try:
-            with codecs.open(timestamp_path, 'rb', 'utf8') as inf:
+            with io.open(timestamp_path, 'r', encoding='utf8') as inf:
                 last_deploy = datetime.strptime(inf.read().strip(), "%Y-%m-%dT%H:%M:%S.%f")
                 clean = False
         except (IOError, Exception) as e:
@@ -96,8 +112,8 @@ class CommandDeploy(Command):
         self._emit_deploy_event(last_deploy, new_deploy, clean, undeployed_posts)
 
         # Store timestamp of successful deployment
-        with codecs.open(timestamp_path, 'wb+', 'utf8') as outf:
-            outf.write(new_deploy.isoformat())
+        with io.open(timestamp_path, 'w+', encoding='utf8') as outf:
+            outf.write(unicode_str(new_deploy.isoformat()))
 
     def _emit_deploy_event(self, last_deploy, new_deploy, clean=False, undeployed=None):
         """ Emit events for all timeline entries newer than last deploy.
@@ -120,9 +136,12 @@ class CommandDeploy(Command):
             'undeployed': undeployed
         }
 
+        if last_deploy.tzinfo is None:
+            last_deploy = last_deploy.replace(tzinfo=gettz('UTC'))
+
         deployed = [
             entry for entry in self.site.timeline
-            if entry.date > last_deploy.replace(tzinfo=self.site.tzinfo) and entry not in undeployed
+            if entry.date > last_deploy and entry not in undeployed
         ]
 
         event['deployed'] = deployed
