@@ -40,7 +40,7 @@ import json
 import shutil
 import subprocess
 import sys
-from zipfile import ZipFile as zip
+from zipfile import ZipFile as zipf
 try:
     from imp import reload
 except ImportError:
@@ -85,6 +85,8 @@ STDERR_HANDLER = [ColorfulStderrHandler(
 LOGGER = get_logger('Nikola', STDERR_HANDLER)
 STRICT_HANDLER = ExceptionHandler(ApplicationWarning, level='WARNING')
 
+USE_SLUGIFY = True
+
 # This will block out the default handler and will hide all unwanted
 # messages, properly.
 logbook.NullHandler().push_application()
@@ -170,6 +172,7 @@ else:
 from doit import tools
 from unidecode import unidecode
 from pkg_resources import resource_filename
+from nikola import filters as task_filters
 
 import PyRSS2Gen as rss
 
@@ -668,11 +671,11 @@ def remove_file(source):
 # slugify is copied from
 # http://code.activestate.com/recipes/
 # 577257-slugify-make-a-string-usable-in-a-url-or-filename/
-_slugify_strip_re = re.compile(r'[^\w\s-]')
+_slugify_strip_re = re.compile(r'[^+\w\s-]')
 _slugify_hyphenate_re = re.compile(r'[-\s]+')
 
 
-def slugify(value):
+def slugify(value, force=False):
     """
     Normalizes string, converts to lowercase, removes non-alpha characters,
     and converts spaces to hyphens.
@@ -691,11 +694,26 @@ def slugify(value):
     """
     if not isinstance(value, unicode_str):
         raise ValueError("Not a unicode object: {0}".format(value))
-    value = unidecode(value)
-    # WARNING: this may not be python2/3 equivalent
-    # value = unicode(_slugify_strip_re.sub('', value).strip().lower())
-    value = str(_slugify_strip_re.sub('', value).strip().lower())
-    return _slugify_hyphenate_re.sub('-', value)
+    if USE_SLUGIFY or force:
+        # This is the standard state of slugify, which actually does some work.
+        # It is the preferred style, especially for Western languages.
+        value = unidecode(value)
+        value = str(_slugify_strip_re.sub('', value).strip().lower())
+        return _slugify_hyphenate_re.sub('-', value)
+    else:
+        # This is the “disarmed” state of slugify, which lets the user
+        # have any character they please (be it regular ASCII with spaces,
+        # or another alphabet entirely).  This might be bad in some
+        # environments, and as such, USE_SLUGIFY is better off being True!
+
+        # We still replace some characters, though.  In particular, we need
+        # to replace ? and #, which should not appear in URLs, and some
+        # Windows-unsafe characters.  This list might be even longer.
+        rc = '/\\?#"\'\r\n\t*:<>|"'
+
+        for c in rc:
+            value = value.replace(c, '-')
+        return value
 
 
 def unslugify(value, discard_numbers=True):
@@ -722,17 +740,18 @@ def extract_all(zipfile, path='themes'):
     pwd = os.getcwd()
     makedirs(path)
     os.chdir(path)
-    with zip(zipfile) as z:
-        namelist = z.namelist()
-        for f in namelist:
-            if f.endswith('/') and '..' in f:
-                raise UnsafeZipException('The zip file contains ".." and is '
-                                         'not safe to expand.')
-        for f in namelist:
-            if f.endswith('/'):
-                makedirs(f)
-            else:
-                z.extract(f)
+    z = zipf(zipfile)
+    namelist = z.namelist()
+    for f in namelist:
+        if f.endswith('/') and '..' in f:
+            raise UnsafeZipException('The zip file contains ".." and is '
+                                     'not safe to expand.')
+    for f in namelist:
+        if f.endswith('/'):
+            makedirs(f)
+        else:
+            z.extract(f)
+    z.close()
     os.chdir(pwd)
 
 
@@ -768,7 +787,7 @@ def current_time(tzinfo=None):
     return dt
 
 
-def apply_filters(task, filters):
+def apply_filters(task, filters, skip_ext=None):
     """
     Given a task, checks its targets.
     If any of the targets has a filter that matches,
@@ -776,6 +795,12 @@ def apply_filters(task, filters):
     and the filter itself to the uptodate of the task.
     """
 
+    if '.php' in filters.keys():
+        if task_filters.php_template_injection not in filters['.php']:
+            filters['.php'].append(task_filters.php_template_injection)
+    else:
+        filters['.php'] = [task_filters.php_template_injection]
+
     def filter_matches(ext):
         for key, value in list(filters.items()):
             if isinstance(key, (tuple, list)):
@@ -789,6 +814,8 @@ def apply_filters(task, filters):
 
     for target in task.get('targets', []):
         ext = os.path.splitext(target)[-1].lower()
+        if skip_ext and ext in skip_ext:
+            continue
         filter_ = filter_matches(ext)
         if filter_:
             for action in filter_:
@@ -897,6 +924,11 @@ def get_asset_path(path, themes, files_folders={'files': ''}, _themes_dir='theme
     return None
 
 
+class LocaleBorgUninitializedException(Exception):
+    def __init__(self):
+        super(LocaleBorgUninitializedException, self).__init__("Attempt to use LocaleBorg before initialization")
+
+
 class LocaleBorg(object):
     """
     Provides locale related services and autoritative current_lang,
@@ -931,6 +963,9 @@ class LocaleBorg(object):
     Examples: "Spanish", "French" can't do the full circle set / get / set
     That used to break calendar, but now seems is not the case, with month at least
     """
+
+    initialized = False
+
     @classmethod
     def initialize(cls, locales, initial_lang):
         """
@@ -965,7 +1000,7 @@ class LocaleBorg(object):
 
     def __init__(self):
         if not self.initialized:
-            raise Exception("Attempt to use LocaleBorg before initialization")
+            raise LocaleBorgUninitializedException()
         self.__dict__ = self.__shared_state
 
     def set_locale(self, lang):
@@ -1003,6 +1038,25 @@ class LocaleBorg(object):
         return s
 
 
+class ExtendedRSS2(rss.RSS2):
+    xsl_stylesheet_href = None
+
+    def publish(self, handler):
+        if self.xsl_stylesheet_href:
+            handler.processingInstruction("xml-stylesheet", 'type="text/xsl" href="{0}" media="all"'.format(self.xsl_stylesheet_href))
+        # old-style class in py2
+        rss.RSS2.publish(self, handler)
+
+    def publish_extensions(self, handler):
+        if self.self_url:
+            handler.startElement("atom:link", {
+                'href': self.self_url,
+                'rel': "self",
+                'type': "application/rss+xml"
+            })
+            handler.endElement("atom:link")
+
+
 class ExtendedItem(rss.RSSItem):
 
     def __init__(self, **kw):
